==> emelfm2-0.1.5.tar.gz: emelfm2-0.1.5/plugins/e2p_unpack.c <==
/* $Id: e2p_unpack.c 1479 2006-01-11 03:42:34Z tpgww $
 *
 * Copyright (C) 2003-2005 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
//#include <unistd.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_task.h"
#include "e2_command.h"
#include "e2_filelist.h"

static gchar *unpack_tmp;	//absolute path of 'working' directory, utf-8 string
static gint thispid;	//id of process which is re-packing an archive

static void _e2p_unpack_retain (gchar *package)
{
  g_free (package);
}

static void _e2p_unpack_clear (gchar *package)
{
  gchar *local = F_FILENAME_TO_LOCALE (unpack_tmp);
  if (e2_fs_access2 (local) == 0)
      e2_task_backend_delete (local);
  F_FREE (local);
  g_free (package);

  e2_filelist_check_dirty (GINT_TO_POINTER(1));
}

//timer function for checking whether a repack is completed
static gboolean _e2p_unpack_clean_dir (gchar *package)
{
  if (e2_command_find_process (thispid))
	return TRUE;	//wait some more
  //now we're ready for cleanup
  _e2p_unpack_clear (package);
  return FALSE;
}

//package is assumed to have ascii-coded extension - ok ?
static void _e2p_unpack_repack (gchar *package)
{
  gchar *command;
  //command strings are all designed to be executed from
  //the temp dir, on all its contents
  if ((strstr (package, ".tar.gz") != NULL) ||
      (strstr (package, ".tgz") != NULL))
    command = ">tar cf - . | gzip - > \"%s\"";
  else if (strstr (package, ".tar.bz2") != NULL)
    command = ">tar cf - . | bzip2 - > \"%s\"";
  else if (strstr (package, ".tar") != NULL)
    command = "tar cf \"%s\" .";
  else if (strstr (package, ".zip") != NULL)
    command = "zip -r \"%s\" .";
  else if (strstr (package, ".rar") != NULL)
	command = "rar u -ol \"%s\" .";
  else //if (strstr (package, ".arj") != NULL)
	command = "arj u -al \"%s\" .";
  command = g_strdup_printf (command, package);

 //just chdir not enough, crashes ...
  gchar *dirnow = g_strdup (curr_view->dir);
  e2_pane_change_dir (NULL, unpack_tmp);
  
  //prevent refreshes in case the inactive pane shows the package
  e2_filelist_disable_refresh ();  //or make it refresh less often (until the background command is completed)
//  gint 
  thispid = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
  e2_filelist_enable_refresh ();  //or resume normal refresh rate

  g_free (command);

  e2_pane_change_dir (NULL, dirnow);
  g_free (dirnow);

  //periodically check whether re-build finished, when so, cleanup the temp dir
  g_timeout_add (500, (GSourceFunc) _e2p_unpack_clean_dir, package);
}

static void _e2p_unpack_response_decode_cb
	(GtkWidget *button, gint response, gchar *package)
{
  switch (response)
  {
	case E2_RESPONSE_USER1:
		_e2p_unpack_repack (package);
		break;
	case E2_RESPONSE_USER2:	
		_e2p_unpack_retain (package);
//	case GTK_RESPONSE_CANCEL:
		break;
	case E2_RESPONSE_REMOVE:
	default:	//this will pick up GTK_RESPONSE_NONE or GTK_RESPONSE_DELETE_EVENT
		_e2p_unpack_clear (package);
		break;
  }
}

//this is a timer callback
static gboolean _e2p_unpack_check_dir (gchar *package)
{
  if (
	 ( g_str_has_prefix (curr_view->dir, unpack_tmp)
	  || g_str_has_prefix (other_view->dir, unpack_tmp))
     )
	  return TRUE;	//continue timer
	//user changed dir to somewhere not in unpack-dir tree
/*    //change the active pane back to the directory where the package is
    FIXME this approach sucks - user changed deliberately, did not expect
	to go back to package dir !!
	ALSO the change seems to confuse gamin
    gchar *s;
    gchar *dir = g_strdup (package);	//package is utf-8
    if ((s = strrchr (dir, G_DIR_SEPARATOR)) != NULL)
    {
      *s = '\0';
      e2_pane_change_dir (NULL, dir);	//change current pane
    }
    g_free (dir);
*/
    //ask user what to do with the unpacked items
  GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
	_("What do you want to do with the unpacked item(s) ?"), NULL,
	_e2p_unpack_response_decode_cb, package);
// no need for a "delete_event" callback, the run-dialog function blocks such events
  e2_dialog_add_undefined_button (dialog, GTK_STOCK_CLEAR,
	_("Re_pack"), E2_RESPONSE_USER1);
  e2_dialog_add_undefined_button (dialog, GTK_STOCK_APPLY,
	_("_Retain"), E2_RESPONSE_USER2);
  GtkWidget *button = e2_dialog_add_undefined_button (dialog, GTK_STOCK_DELETE,
	_("_Delete"), E2_RESPONSE_REMOVE);

  gtk_widget_grab_focus (button);
	//as a timer, this func is outside gtk's lock, so need explicit thread protection
  e2_dialog_show (dialog, app.main_window,
	E2_DIALOG_RUN | E2_DIALOG_THREADED | E2_DIALOG_FREE, NULL);
    return FALSE;	//stop the timer
}

static void _e2p_unpack (void)
{
  gchar *package;	//absolute path of archive, utf8 string
  gchar *command;	//unpack-command string
  FileInfo *info;

  info = e2_fileview_get_selected_first_local (curr_view);
  if (info == NULL)
	  return;	//nothing selected

  if (strstr (curr_view->dir, unpack_tmp) != NULL)
  { //the temp dir is deleted when this plugin function is
	 //performed, so It would be very bad to open an archive
	 //inside the temp dir
    e2_output_print_error (_("Recursive unpack is not supported"));
    return;
  }

  //delete the temp dir if it exists already
  gchar *converted = F_FILENAME_TO_LOCALE (unpack_tmp);
  if (e2_fs_access2 (converted) == 0)
  {
	  gchar *saved = e2_utils_get_tempname (converted);
      e2_task_backend_rename (converted, saved);
	  g_free (saved);
  }
  //(re)make it
  if (e2_fs_mkdir (converted, 0777))
  {
	gchar *msg = g_strdup_printf ("Could not create working directory '%s'",
		unpack_tmp);
	e2_output_print_error (msg);
	g_free (msg);
	F_FREE (converted);
	return;
  }
  F_FREE (converted);

  //note: --atime-preserve in these tar commands prevents the
  //file-monitoring process from noticing anything in the temp dir
  //various strstr's all applied to localised strings
  if ((strstr (info->filename, ".tar.gz") != NULL) ||
      (strstr (info->filename, ".tgz") != NULL))
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
    command = "tar -xpzf \"%s\"";
#else
   //FIXME -z option is a gnu extension
    command = "tar --overwrite -xpzf \"%s\"";
#endif
  else if (strstr (info->filename, ".tar.bz2") != NULL)
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
    command = "tar -xpjf \"%s\"";
#else
   //FIXME -j option is a gnu extension
    command = "tar --overwrite -xpjf \"%s\"";
 #endif
  else if (strstr (info->filename, ".tar") != NULL)
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
    command = "tar -xpf \"%s\"";
#else
    command = "tar --overwrite -xpf \"%s\"";
#endif
  //.deb command (for what version ?) provided by Martin Zelaia
//  else if (strstr(info->filename, ".deb") != NULL)
//    command = "mkdir ./DEBIAN ./CONTENTS;>ar -x \"%s\" | tar -xfz control.tar.gz -C ./DEBIAN | tar -xfz data.tar.gz -C ./CONTENTS; rm control.tar.gz data.tar.gz;cp ./DEBIAN/control ./INFO;rm ./debian-binary";
  else if (strstr (info->filename, ".zip") != NULL)
    command = "unzip -o \"%s\"";
  else if (strstr(info->filename, ".rar") != NULL)
    command = "rar x -o+ \"%s\"";
  else if (strstr(info->filename, ".arj") != NULL)
    command = "arj x -y \"%s\"";  
  else
  {
    e2_output_print_error (_("Selected item is not a supported archive"));
    return;
  }

  converted = F_FILENAME_FROM_LOCALE (info->filename);
  package = g_strdup_printf ("%s%s", curr_view->dir, converted);  //dir has trailing /
  command = g_strdup_printf (command, package);

  e2_widget_set_cursor (GDK_WATCH);

  //after info is parsed,
  //we need to go to the temp dir to execute the command
  e2_pane_change_dir (NULL, unpack_tmp);
  e2_filelist_disable_refresh (); //prevent nested updates when we check-dirty()

  //unpack the archive into the temp directory
  e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
  //update pane
  e2_filelist_check_dirty (GINT_TO_POINTER(1));
  e2_filelist_enable_refresh ();

  e2_widget_set_cursor (GDK_LEFT_PTR);

  F_FREE (converted);
  g_free (command);

  //periodically check whether to cleanup the unpacked things
  g_timeout_add (500, (GSourceFunc) _e2p_unpack_check_dir, package);
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
  //once-only, setup the working dir name
//  if (unpack_tmp == NULL)
// {
	const gchar *tmp = g_getenv ("TMPDIR");
	if (tmp != NULL)
	 unpack_tmp = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s-unpack", tmp, BINNAME);
	else
	{
#ifdef E2_FAM
	  unpack_tmp = g_strconcat (e2_cl_options.config_dir, 
	  G_DIR_SEPARATOR_S"unpack", NULL);
#else
	//don't put into config dir, that triggers config refreshing
	//in shared space, need temp-dir for each user, in case the dir is left there after unpacking
	gint myuid = getuid ();
	  unpack_tmp = g_strdup_printf (G_DIR_SEPARATOR_S"tmp"G_DIR_SEPARATOR_S"%s-unpack-%d", BINNAME, myuid);
#endif
	}
// }

#define ANAME "unpack"
  aname = _("unpack");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Unpack..");
  p->description = _("Unpack archive (tar, tar.gz, tar.bz2, zip, rar, arj) into a temporary directory");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	  p->plugin_cb = _e2p_unpack;  //set plugins menu item function
//	  E2_Action* (*action_register) () = (*p->api_lookup) (E2API_ACTION_REGISTER);
	  //no need to free this
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
	  //NULL = no data, FALSE = no argument for the function
//	  (*action_register)
	  e2_action_register_simple (action_name, E2_ACTION_TYPE_ITEM, _e2p_unpack, NULL, FALSE);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief unload this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return
 **/
void unload_plugin (Plugin *p)
{
//  if (p->api_lookup != NULL)
//  {
//	  gboolean (*action_unregister) () = (*p->api_lookup) (E2API_ACTION_UNREGISTER);
	  gchar *action_name = g_strconcat (PLUGIN,".",aname,NULL);
//	  (*action_unregister)
	  e2_action_unregister (action_name);
	  g_free (action_name);
//  }
}

==> emelfm2-0.1.7.tar.gz: emelfm2-0.1.7/plugins/e2p_unpack.c <==
/* $Id: e2p_unpack.c 1856 2006-04-24 12:51:28Z tpgww $
 *
 * Copyright (C) 2003-2006 tooar <tooar@gmx.net>
 *
 * This file is part of emelFM2.
 * emelFM2 is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * emelFM2 is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with emelFM2; see the file GPL. If not, contact the Free Software
 * Foundation, 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */

#include "emelfm2.h"
#include <string.h>
//#include <unistd.h>
#include "e2_plugins.h"
#include "e2_dialog.h"
#include "e2_task.h"
#include "e2_command.h"
#include "e2_filelist.h"

typedef struct
{
  gchar *package;	////absolute path of source archive, utf8 string
  gint thispid;		//id of process which is re-packing an archive
  guint chdir_id;	//id of timer which checks for whether to ask user
  guint pack_id;	//id of timer which checks for repack completed
} E2P_Unpackdata;

static gchar *unpack_tmp;	//absolute path of 'working' directory, utf-8 string

static void _e2p_unpack_retain (E2P_Unpackdata *data)
{
  g_free (data->package);
  g_free (data);
}

static void _e2p_unpack_clear (E2P_Unpackdata *data)
{
  gchar *local = F_FILENAME_TO_LOCALE (unpack_tmp);
  if (e2_fs_access2 (local) == 0)
      e2_task_backend_delete (local);
  F_FREE (local);
  g_free (data->package);
  g_free (data);
  e2_filelist_check_dirty (GINT_TO_POINTER(1));
}

//timer function for checking whether a repack is completed
static gboolean _e2p_unpack_clean_dir (E2P_Unpackdata *data)
{
  if (e2_command_find_process (data->thispid))
	return TRUE;	//wait some more
  //now we're ready for cleanup
  g_source_remove (data->pack_id);
  _e2p_unpack_clear (data);
  return FALSE;
}

//package is assumed to have ascii-coded extension - ok ?
static void _e2p_unpack_repack (E2P_Unpackdata *data)
{
  gchar *command, *package = data->package;
  //command strings are all designed to be executed from
  //the temp dir, on all its contents
  if ((strstr (package, ".tar.gz") != NULL) ||
      (strstr (package, ".tgz") != NULL))
    command = ">tar cf - . | gzip - > \"%s\"";
  else if (strstr (package, ".tar.bz2") != NULL)
    command = ">tar cf - . | bzip2 - > \"%s\"";
  else if (strstr (package, ".tar") != NULL)
    command = "tar cf \"%s\" .";
  else if (strstr (package, ".zip") != NULL)
    command = "zip -r \"%s\" .";
  else if (strstr (package, ".rar") != NULL)
	command = "rar u -ol \"%s\" .";
  else //if (strstr (package, ".arj") != NULL)
	command = "arj u -al \"%s\" .";
  command = g_strdup_printf (command, package);

 //just chdir not enough, crashes ...
  gchar *dirnow = g_strdup (curr_view->dir);
  e2_pane_change_dir (NULL, unpack_tmp);
  
  //prevent refreshes in case the inactive pane shows the package
  e2_filelist_disable_refresh ();  //or make it refresh less often (until the background command is completed)
  gint res = e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
  data->thispid = (res == 0) ? e2_command_find_last_child () : 0;
  e2_filelist_enable_refresh ();  //or resume normal refresh rate

  g_free (command);

  e2_pane_change_dir (NULL, dirnow);
  g_free (dirnow);

  //periodically check whether re-build finished, when so, cleanup the temp dir
  data->pack_id = g_timeout_add (500, (GSourceFunc) _e2p_unpack_clean_dir, data);
}

static void _e2p_unpack_response_decode_cb
	(GtkWidget *button, gint response, E2P_Unpackdata *data)
{
  switch (response)
  {
	case E2_RESPONSE_USER1:
		_e2p_unpack_repack (data);
		break;
	case E2_RESPONSE_USER2:	
		_e2p_unpack_retain (data);
//	case GTK_RESPONSE_CANCEL:
		break;
	case E2_RESPONSE_REMOVE:
	default:	//this will pick up GTK_RESPONSE_NONE or GTK_RESPONSE_DELETE_EVENT
		_e2p_unpack_clear (data);
		break;
  }
}

//this is a timer callback
static gboolean _e2p_unpack_check_dir (E2P_Unpackdata *data)
{
  if (
	 ( g_str_has_prefix (curr_view->dir, unpack_tmp)
	  || g_str_has_prefix (other_view->dir, unpack_tmp))
     )
	  return TRUE;	//continue timer
	//user changed dir to somewhere not in unpack-dir tree
/*    //change the active pane back to the directory where the package is
    FIXME this approach sucks - user changed deliberately, did not expect
	to go back to package dir !!
	ALSO the change seems to confuse gamin
    gchar *s;
    gchar *dir = g_strdup (package);	//package is utf-8
    if ((s = strrchr (dir, G_DIR_SEPARATOR)) != NULL)
    {
      *s = '\0';
      e2_pane_change_dir (NULL, dir);	//change current pane
    }
    g_free (dir);
*/
  if (data->chdir_id > 0)
	  //cancel the timer asap in case of race when deleting
	  g_source_remove (data->chdir_id);

    //ask user what to do with the unpacked items
  GtkWidget *dialog = e2_dialog_create (GTK_STOCK_DIALOG_QUESTION,
	_("What do you want to do with the unpacked item(s) ?"), NULL,
	_e2p_unpack_response_decode_cb, data);
// no need for a "delete_event" callback, the run-dialog function blocks such events
  e2_dialog_add_undefined_button (dialog, GTK_STOCK_CLEAR,
	_("Re_pack"), E2_RESPONSE_USER1);
  e2_dialog_add_undefined_button (dialog, GTK_STOCK_APPLY,
	_("_Retain"), E2_RESPONSE_USER2);
  GtkWidget *button = e2_dialog_add_undefined_button (dialog, GTK_STOCK_DELETE,
	_("_Delete"), E2_RESPONSE_REMOVE);

  gtk_widget_grab_focus (button);
	//as a timer, this func is outside gtk's lock, so need explicit thread protection
  e2_dialog_show (dialog, NULL,
	E2_DIALOG_RUN | E2_DIALOG_THREADED | E2_DIALOG_FREE, NULL);
    return FALSE;	//stop the timer
}
/**
 * @brief unpack a supported archive into a temp dir
 *
 * @param action_data ptr to data assigned when action struct created at session start
 * @param rt_data ptr to data sent to this command, NULL to act on selection
 * @param widget optional pointer to widget where the action was initiated, generally NULL
 *
 * @return TRUE if action completed successfully, else FALSE
 */
static gboolean _e2p_unpack (gpointer action_data, gpointer rt_data, GtkWidget *widget)
{
  gchar *command;	//unpack-command string
  FileInfo *info;

  info = e2_fileview_get_selected_first_local (curr_view);
  if (info == NULL)
	  return FALSE;	//nothing selected

  if (strstr (curr_view->dir, unpack_tmp) != NULL)
  { //the temp dir may be deleted when the plugin function is
	 //completed, so it would be very bad to open an archive
	 //inside the temp dir
    e2_output_print_error (_("Recursive unpack is not supported"), FALSE);
    return FALSE;
  }

  //delete the temp dir if it exists already
  gchar *converted = F_FILENAME_TO_LOCALE (unpack_tmp);
  if (e2_fs_access2 (converted) == 0)
  {
	  gchar *saved = e2_utils_get_tempname (converted);
     e2_task_backend_rename (converted, saved);
	  g_free (saved);
  } 
  //(re)make it
  if (e2_fs_mkdir (converted, 0777))
  {
	gchar *msg = g_strdup_printf ("Could not create working directory '%s'",
		unpack_tmp);
	e2_output_print_error (msg, TRUE);
	F_FREE (converted);
	return FALSE;
  }
  F_FREE (converted);

  //note: --atime-preserve in these tar commands prevents the
  //file-monitoring process from noticing anything in the temp dir
  //various strstr's all applied to localised strings
  if ((strstr (info->filename, ".tar.gz") != NULL) ||
      (strstr (info->filename, ".tgz") != NULL))
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
    command = "tar -xpzf \"%s\"";
#else
   //FIXME -z option is a gnu extension
    command = "tar --overwrite -xpzf \"%s\"";
#endif
  else if (strstr (info->filename, ".tar.bz2") != NULL)
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
    command = "tar -xpjf \"%s\"";
#else
   //FIXME -j option is a gnu extension
    command = "tar --overwrite -xpjf \"%s\"";
 #endif
  else if (strstr (info->filename, ".tar") != NULL)
#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(darwin)
    command = "tar -xpf \"%s\"";
#else
    command = "tar --overwrite -xpf \"%s\"";
#endif
  //.deb command (for what version ?) provided by Martin Zelaia
//  else if (strstr(info->filename, ".deb") != NULL)
//    command = "mkdir ./DEBIAN ./CONTENTS;>ar -x \"%s\" | tar -xfz control.tar.gz -C ./DEBIAN | tar -xfz data.tar.gz -C ./CONTENTS; rm control.tar.gz data.tar.gz;cp ./DEBIAN/control ./INFO;rm ./debian-binary";
  else if (strstr (info->filename, ".zip") != NULL)
    command = "unzip -o \"%s\"";
  else if (strstr(info->filename, ".rar") != NULL)
    command = "rar x -o+ \"%s\"";
  else if (strstr(info->filename, ".arj") != NULL)
    command = "arj x -y \"%s\"";  
  else
  {
    e2_output_print_error (_("Selected item is not a supported archive"), FALSE);
    return FALSE;
  }

  converted = F_FILENAME_FROM_LOCALE (info->filename);
  gchar *package = g_strdup_printf ("%s%s", curr_view->dir, converted);  //dir has trailing /
  command = g_strdup_printf (command, package);

  E2P_Unpackdata *data = (E2P_Unpackdata *) g_malloc (sizeof (E2P_Unpackdata));
  data->package = package;
  
  e2_widget_set_cursor (GDK_WATCH);

  //after info is parsed,
  //we need to go to the temp dir to execute the command
  e2_pane_change_dir (NULL, unpack_tmp);
  e2_filelist_disable_refresh (); //prevent nested updates when we check-dirty()

  //unpack the archive into the temp directory
  e2_command_run (command, E2_COMMAND_RANGE_DEFAULT);
  //update pane
  e2_filelist_check_dirty (GINT_TO_POINTER(1));
  e2_filelist_enable_refresh ();

  e2_widget_set_cursor (GDK_LEFT_PTR);

  F_FREE (converted);
  g_free (command);

  //periodically check whether to cleanup the unpacked things
  data->chdir_id = g_timeout_add (500, (GSourceFunc) _e2p_unpack_check_dir, data);
  return TRUE;
}

//aname must be confined to this module
static gchar *aname;
/**
 * @brief plugin initialization function, called by main program
 *
 * @param p ptr to plugin data struct
 *
 * @return TRUE if the initialization succeeds, else FALSE
 **/
gboolean init_plugin (Plugin *p)
{
  //once-only, setup the working dir name
  const gchar *tmp = g_get_tmp_dir ();
  if (g_str_has_prefix (tmp, g_get_home_dir ()))
	  unpack_tmp = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s-unpack", tmp, BINNAME);
  else
  {
  //in shared space, need temp-dir for each user, in case the dir is left there after unpacking
	gint myuid = getuid ();
	unpack_tmp = g_strdup_printf ("%s"G_DIR_SEPARATOR_S"%s-unpack-%d", tmp, BINNAME, myuid);
  }

#define ANAME "unpack"
  aname = _("unpack");

  p->signature = ANAME VERSION;
  p->menu_name = _("_Unpack..");
  p->description = _("Unpack archive (tar, tar.gz, tar.bz2, zip, rar, arj) into a temporary directory");
  p->icon = "plugin_"ANAME"_"E2IP".png";  //use icon file pathname if appropriate

  if (p->plugin_cb == NULL)
  {
	  p->plugin_cb = _e2p_unpack;  //set plugins menu item function
	  //no need to free this
	  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
	  e2_plugins_action_register (action_name, E2_ACTION_TYPE_ITEM,
		  _e2p_unpack, NULL, FALSE, 0, NULL);
	  return TRUE;
  }
  return FALSE;
}
/**
 * @brief cleanup transient things for this plugin
 *
 * @param p pointer to data struct for the plugin
 *
 * @return TRUE if all cleanups were completed
 **/
gboolean clean_plugin (Plugin *p)
{
//FIXME prevent unload when an unpack is underway
  gchar *action_name = g_strconcat (_A(5),".",aname,NULL);
  gboolean ret = e2_plugins_action_unregister (action_name);
  g_free (action_name);
  return ret;
}
